/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client.monitors;

//---IMPORTS---
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Iterator;
import java.text.NumberFormat;
import uk.org.iscream.cms.server.client.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;

/**
 * This Monitor watches the Disks for all machines
 *
 * @author  $Author: tdb $
 * @version $Id: Disk__Monitor.java,v 1.19 2003/03/09 21:49:13 tdb Exp $
 */
public class Disk__Monitor extends MonitorSkeleton {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.19 $";
    
    /**
     * A description of this monitor
     */
    public final String DESC = "Monitors all host disks.";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

//---PUBLIC METHODS---
    
    /**
     * Analyse a packet of data, and generate an alert if
     * necessary.
     *
     * @param packet the XMLPacket to analyse
     */
    public void analysePacket(XMLPacket packet) {
        String source = packet.getParam("packet.attributes.machine_name");
        if (!_hosts.containsKey(source)) {
            _hosts.put(source, new HashMap());
        }
        if (!_hostsInode.containsKey(source)) {
            _hostsInode.put(source, new HashMap());
        }

        HashMap diskRegisters = (HashMap) _hosts.get(source);
        HashMap diskInodeRegisters = (HashMap) _hostsInode.get(source);
        
        // key prefix
        String keyPrefix = "packet.disk.p";
        
        // a temporary holder for all the disk attributes we find
        HashSet disks = new HashSet();

        // unfortunatly we need to check the whole packet
        // to find the disks, and then get the data attributes
        Set packetSet = packet.getSet();
        Iterator i = packetSet.iterator();
        while (i.hasNext()) {
            String dataKey = (String) i.next();
            if(dataKey.startsWith(keyPrefix)) {
                // pos is after "packet.disk.p"
                int pos = keyPrefix.length();
                String diskNumber = dataKey.substring(pos, dataKey.indexOf('.', pos));
                String thisDisk = keyPrefix.concat(diskNumber);
                
                if(!disks.contains(thisDisk)) {
                    // add the disk to our list
                    disks.add(thisDisk);

                    // device
                    String device = packet.getParam(thisDisk + ".attributes.name");
                    // mount point
                    String mount = packet.getParam(thisDisk + ".attributes.mount");

                    // these next two will be used to calculate the %age
                    // total
                    String total = packet.getParam(thisDisk + ".attributes.total");
                    // available
                    String avail = packet.getParam(thisDisk + ".attributes.avail");
                    
                    String totalInodes = packet.getParam(thisDisk + ".attributes.totalinodes");
                    String availInodes = packet.getParam(thisDisk + ".attributes.freeinodes");

                    // *** now process this disk ***

                    // check if we've seen this disk before on a previous run
                    // if not, we need to create a register for it
                    //   nb. use the mount as the key as this is unlikely to change,
                    //       unlike diskNumber which could easily change
                    //         (diskNumber is based on the order of df's output!)
                    if(!diskRegisters.containsKey(mount)) {
                        diskRegisters.put(mount, new Register(source, _name, mount));
                    }
                    if(!diskInodeRegisters.containsKey(mount)) {
                        diskInodeRegisters.put(mount, new Register(source, _nameInode, mount));
                    }

                    // get the register for this disk
                    Register reg = (Register) diskRegisters.get(mount);
                    Register regInode = (Register) diskInodeRegisters.get(mount);

                    // get the packet data
                    double diskTotal, diskAvail;
                    int diskInodeTotal, diskInodeAvail;
                    try {

                        if(total==null || avail==null || totalInodes==null || availInodes==null) {
                            throw new NumberFormatException("Disk data invalid");
                        }
                        diskTotal = Double.parseDouble(total);
                        diskAvail = Double.parseDouble(avail);
                        diskInodeTotal = Integer.parseInt(totalInodes);
                        diskInodeAvail = Integer.parseInt(availInodes);
                    } catch (NumberFormatException e) {
                        _logger.write(this.toString(), Logger.WARNING, "Received packet from "+source+" with bad disk information: "+e);
                        // don't try to continue and process, try next disk
                        break;
                    }

                    boolean useValue = false;
                    boolean useValueInode = false;
                    try {
                        // try looking for a mount-point specific thresholdMeasure first
                        String option = _cp.getProperty("Host." + source, "Monitor." + _name + "." + mount + ".thresholdMeasure");
                        if (option.equals("VALUE")) {
                            useValue = true;
                        }
                        option = _cp.getProperty("Host." + source, "Monitor." + _nameInode + "." + mount + ".thresholdMeasure");
                        if (option.equals("VALUE")) {
                            useValueInode = true;
                        }
                    } catch (PropertyNotFoundException e) {
                        try {
                            // now look for a more general thresholdMeasure
                            String option = _cp.getProperty("Host." + source, "Monitor." + _name + ".thresholdMeasure");
                            if (option.equals("VALUE")) {
                                useValue = true;
                            }
                            option = _cp.getProperty("Host." + source, "Monitor." + _nameInode + ".thresholdMeasure");
                            if (option.equals("VALUE")) {
                                useValueInode = true;
                            }
                        } catch (PropertyNotFoundException f) {
                            // we default to percentage in any case
                        }
                    }

                    // this  bit determines if the disk check is a % check
                    // or a byte check
                    String type;
                    double curValue;
                    int newThreshold;
                    if(useValue) {
                        // byte disk available
                        curValue = diskAvail;
                        // negate check
                        newThreshold = checkAttributeThreshold(curValue, reg, true);
                        type = "bytes";
                    } else {
                        // % disk in use
                        curValue = (1 - ((double)diskAvail / (double)diskTotal)) * 100;
                        // normal check
                        newThreshold = checkAttributeThreshold(curValue, reg, false);
                        type = "%";
                    }
                    
                    String typeInode;
                    double curValueInode;
                    int newThresholdInode;
                    if(useValueInode) {
                        // inodes available
                        curValueInode = diskInodeAvail;
                        // negate check
                        newThresholdInode = checkAttributeThreshold(curValueInode, regInode, true);
                        typeInode = "count";
                    } else {
                        // % inodes in use
                        curValueInode = (1 - ((double)diskInodeAvail / (double)diskInodeTotal)) * 100;
                        // normal check
                        newThresholdInode = checkAttributeThreshold(curValueInode, regInode, false);
                        typeInode = "%";
                    }
                    
                    // format the diskInUse to a String
                    NumberFormat nf = NumberFormat.getInstance();
                    nf.setMaximumFractionDigits(2);
                    nf.setMinimumFractionDigits(2);
                    String strCurValue = nf.format(curValue);
                    String strCurValueInode = nf.format(curValueInode);

                    // say which disk had the problem
                    String attributeName = "Disk in use " + type + " on " + mount + " (" + device + ")";
                    String attributeNameInode = "Disk inodes in use " + type + " on " + mount + " (" + device +")";

                    processAlert(newThreshold, attributeName, reg, source, strCurValue);
                    processAlert(newThresholdInode, attributeNameInode, regInode, source, strCurValueInode);
                }
            }
        }
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

    /**
     * return the String representation of what the monitor does
     */
    public String getDescription(){
        return DESC;
    }

//---PRIVATE METHODS---
    
    /**
     * Checks a piece of current data, and returns the
     * threshold it breaches, if any.
     *
     * The option to negate the check can be used in
     * situations where being *below* the threshold
     * is an 'alertable' situation. In this specific
     * case, we'd do this with kb disk checks.
     *
     * @param value the current value
     * @param reg the Register for the host
     * @param negateCheck whether to negate the check
     * @return the threshold level breached, if any
     */
    private int checkAttributeThreshold(double value, Register reg, boolean negateCheck) {
        for(int thresholdLevel = Alert.thresholdLevels.length - 1; thresholdLevel >= 0; thresholdLevel--) {
            if (reg.getThreshold(thresholdLevel) != -1.0) {
                if(!negateCheck) {
                    // normal check - has the value gone *over* the threshold
                    if(((double) reg.getThreshold(thresholdLevel)) < value) {
                        return thresholdLevel;
                    }
                }
                else {
                    // negated check - has the value gone *under* the threshold
                    if(((double) reg.getThreshold(thresholdLevel)) > value) {
                        return thresholdLevel;
                    }
                }
            }
        }
        return Alert.thresholdNORMAL;
    }

//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Returns a reference to a specific Queue for this
     * monitor. This Queue returns only the data packets
     * (based on type) that we want too look at.
     *
     * @return a reference to a Queue
     */
    protected Queue getQueue() {
        return MonitorManager.getInstance().getDataQueue();
    }
    
//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = "Disk";
    
    /**
     * Inodes have the name "DiskInode" in the
     * configuration.
     */
    private String _nameInode = "DiskInode";
    
    /**
     * A reference to the configuration proxy in use
     */
    private ConfigurationProxy _cp = ConfigurationProxy.getInstance();
    
    /**
     * A HashMap of Registers (or groups of Registers), one
     * for each host we're monitoring.
     */
    private HashMap _hosts = new HashMap();
    
    /**
     * A HashMap of Registers (or groups of Registers), one
     * for each host we're monitoring (inodes).
     */
    private HashMap _hostsInode = new HashMap();

//---STATIC ATTRIBUTES---

}
