/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client.monitors;

//---IMPORTS---
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Set;
import java.util.Iterator;
import java.text.NumberFormat;
import uk.org.iscream.cms.server.client.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;

/**
 * This Monitor watches the Service checks on hosts
 *
 * @author  $Author: tdb $
 * @version $Id: Services__Monitor.java,v 1.9 2003/02/24 20:18:48 tdb Exp $
 */
public class Services__Monitor extends MonitorSkeleton {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.9 $";
    
    /**
     * A description of this monitor
     */
    public final String DESC = "Monitors a hosts services.";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

//---PUBLIC METHODS---
    
    /**
     * Analyse a packet of data, and generate an alert if
     * necessary.
     *
     * @param packet the XMLPacket to analyse
     */
    public void analysePacket(XMLPacket packet) {
        String source = packet.getParam("packet.attributes.machine_name");
        if (!_hosts.containsKey(source)) {
            _hosts.put(source, new HashMap());
        }

        HashMap serviceRegisters = (HashMap) _hosts.get(source);

        // a tempory holder for all the disk attributes we find
        ArrayList services = new ArrayList();
        
        // the prefix for keys relating to service checks
        String keyPrefix = "packet.services.";
        
        // unfortunatly we need to check the whole packet
        // to find the disks, and then get the data attributes
        Set packetSet = packet.getSet();
        Iterator i = packetSet.iterator();
        while (i.hasNext()) {
            String dataKey = (String) i.next();
            if(dataKey.startsWith(keyPrefix)) {
                if(!services.contains(dataKey)) {
                    String serviceType = "";
                    
                    // pos is after "packet.services."
                    int pos = keyPrefix.length();
                    while (dataKey.charAt(pos) != '.') {
                        serviceType = serviceType + dataKey.charAt(pos);
                        pos++;
                    }
                    
                    // add the service to our list, with the packet data                       
                    services.add(keyPrefix + serviceType + ".attributes.status");
                    String status = packet.getParam(keyPrefix + serviceType + ".attributes.status");
                    
                    services.add(keyPrefix + serviceType + ".attributes.message");
                    String message = packet.getParam(keyPrefix + serviceType + ".attributes.message");
                    
                    // *** now process this service ***
                    
                    // check if we've seen this service before on a previous run
                    // if not, we need to create a register for it
                    if(!serviceRegisters.containsKey(serviceType)) {
                        serviceRegisters.put(serviceType, new Register(source, _name));
                    }
                    
                    // get the register for this service
                    Register reg = (Register) serviceRegisters.get(serviceType);
                    
                    
                    // as we don't really have a threshold for services,
                    // a temporary fix is to use the status value as a threshold.
                    // after all, a 0 status is a NORMAL threshold ;)
                    int newThreshold = 0;
                    try {
                        newThreshold = Integer.parseInt(status);
                    } catch (NumberFormatException e) {
                        _logger.write(this.toString(), Logger.WARNING, "Received heartbeat from "+source+" with bad service information: "+e);
                        // don't try to continue and go try the next service
                        break;
                    }
                    
                    // say which service it was
                    String attributeName = serviceType + " service";
                    String displayValue = "";
                    if (newThreshold == 0) {
                        displayValue = "RUNNING";
                    } else {
                        displayValue = "FAILED";
                    }
                    
                    processAlert(newThreshold, attributeName, reg, source, displayValue);
                }
            }
        }
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

    /**
     * return the String representation of what the monitor does
     */
    public String getDescription(){
        return DESC;
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Returns a reference to a specific Queue for this
     * monitor. This Queue returns only the data packets
     * (based on type) that we want too look at.
     *
     * @return a reference to a Queue
     */
    protected Queue getQueue() {
        return MonitorManager.getInstance().getOtherQueue();
    }

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = "Services";
    
    /**
     * A HashMap of Registers (or groups of Registers), one
     * for each host we're monitoring.
     */
    private HashMap _hosts = new HashMap();

//---STATIC ATTRIBUTES---

}
