/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.clientinterface;

//---IMPORTS---
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;

/**
 * This class, a servant, listens for incoming connections from CORBA based
 * clients. Once it receives a connection it creates and assigns a CorbaControlHandler
 * object to it. The CorbaControlHandler will then handle all requests on behalf of the
 * client, and will send the client data.
 *
 * @author  $Author: tdb $
 * @version $Id: CorbaClientListenerServant.java,v 1.13 2003/05/05 22:05:06 tdb Exp $
 */
class CorbaClientListenerServant extends CorbaClientListenerPOA {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.13 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Creates a new CorbaClientListenerServant.
     *
     * @param packetSorter a reference to the PacketSorter object being used
     */
    public CorbaClientListenerServant(PacketSorter packetSorter) {
        _packetSorter = packetSorter;
        _logger.write(toString(), Logger.SYSINIT, "created");
    }
    
//---PUBLIC METHODS---

    /**
     * Allows a CORBA client to connect to the server.
     *
     * @param client a reference to the "servant" part of the connecting client.
     * @param name a name to identify the client.
     * @return a CorbaControlHandler reference.
     */
    public CorbaControlHandler connect(Client client, String name) {
        CorbaControlHandlerServant cchServant = new CorbaControlHandlerServant(_packetSorter, client, name);
        org.omg.CORBA.Object objRef;
        try {
            objRef = _refman.getRootPOA().servant_to_reference(cchServant);
        } catch (Exception e) {
            objRef = null;
            _logger.write(toString(), Logger.ERROR, "Error creating CorbaControlHandler: "+e);
        }
        CorbaControlHandler corbaHandler = CorbaControlHandlerHelper.narrow(objRef);
        return corbaHandler;
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = ClientInterfaceMain.NAME;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();
    
    /**
     * A reference to the PacketSorter.
     */
    private PacketSorter _packetSorter;
    
//---STATIC ATTRIBUTES---

}
