/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.core;

//---IMPORTS---
import uk.org.iscream.cms.util.*;

/**
 * The LoggerServant is an implementation of the Logger defined
 * in the IDL interface.  Allows classes to send logging information
 * over CORBA to a local implementation of the LoggerImpl interface.
 *
 * @author  $Author: tdb $
 * @version $Id: LoggerServant.java,v 1.13 2003/02/05 16:43:47 tdb Exp $
 */
class LoggerServant extends LoggerPOA {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.13 $";
    
    /**
     * An array of names of verbosity levels.
     * Thus logging messages are now "classed" by the level"
     * This string is prepended to the "source" of a logging message.
     */
    public final static String[] VERBOSITY_NAMES = {"FATAL", "ERROR", "WARNING", "SYSMSG", "SYSINIT", "DEBUG"};
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Creates a new LoggerServant.
     *
     * @param logger a reference to the LoggerImpl this will use
     */
    public LoggerServant(LoggerImpl logger) {
        _logger = logger;
    	_verbosityLevel = Integer.parseInt(System.getProperty("uk.org.iscream.cms.server.Verbosity"));
        write(toString(), Logger.SYSINIT, "started");
        write(toString(), Logger.SYSMSG, "using verbosity " + _verbosityLevel);
    }

//---PUBLIC METHODS---

    /**
	 * The write() method takes a source, verbosity level and
	 * message, and formats them using an external line
	 * formatting method. This line is then given to the logger
	 * to be written. Note that checking of the level is carried
	 * out here.
	 * 
	 * @param source A string representation of the calling object.
	 * @param verbosity the verbosity of this message
	 * @param message The text to be logged.
	 */   
    public void write(String source, int verbosity, String message) {
        if (verbosity <= _verbosityLevel) {
            _logger.write(FormatName.formatLogLine(source, verbosity, message), verbosity);
        }
    }
       
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }
    
//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

	/**
	 * The verbosity level of this instance
	 */
	private int _verbosityLevel;
	
	/**
	 * The actual Logger used by this instance
	 */
	private LoggerImpl _logger;
    
    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = Core.NAME;
    
//---STATIC ATTRIBUTES---
    
}
