/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.rootfilter;

//---IMPORTS---
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.filter.*;
import uk.org.iscream.cms.util.*;

/**
 * A root filter servant has an array
 * of client interfaces where data should be
 * piped.
 *
 * @author  $Author: tdb $
 * @version $Id: RootFilterServant.java,v 1.21 2003/05/05 22:05:14 tdb Exp $
 */
class RootFilterServant extends FilterPOA {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.21 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Creates a new RootFilter.
     *
     * @param queue a Queue to use
     */
    public RootFilterServant(Queue queue) {
        _queue = queue;
        _logger.write(toString(), Logger.SYSINIT, "created");
    }
 
//---PUBLIC METHODS---

    /**
     * Method to receive a string over corba.
     *
     * @param xml the String of XML to send on
     */
    public void receiveXML(String xml) {
        _queue.add(xml);
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = RootFilter.NAME;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * A reference to the Queue being used.
     */
    private Queue _queue;
    
//---STATIC ATTRIBUTES---

}
